#include <xcore/port.h>
#include <xcore/parallel.h>
#include <xs1.h>

port_t input = XS1_PORT_1B;
port_t output = XS1_PORT_1A;
xclock_t clk = XS1_CLKBLK_1;

// start respond
void respond(port_t q, port_t p) {
    port_set_trigger_in_equal(p, 1);
    port_in(p);
    port_out(q, 1);
    port_out(q, 0);
}
// end respond

// start init
void init_port(port_t p, port_t q, xclock_t clk) {
    port_enable(p);
    port_enable(q);
    clock_enable(clk);
    clock_set_source_clk_xcore(clk);
    port_set_clock(p, clk);
    port_set_clock(q, clk);
    clock_start(clk);
}
// end init

DECLARE_JOB(busymain, (void));
DECLARE_JOB(iomain, (void));

void busymain(void) {
    while(1);
}

// start iomain
void iomain(void) {
    local_thread_mode_set_bits(thread_mode_high_priority);  // Comment out to not have priority mode
    local_thread_mode_set_bits(thread_mode_fast);           // Comment out to not have fast mode
    init_port(input, output, clk);
    port_out(output, 0);
    while(1) {
        respond(output, input);
    }
}
// end iomain

int main(void) {
    PAR_JOBS(
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(busymain, ()),    // comment out to reduce number of busy threads
        PJOB(iomain, ()));
}
