// cc -g -m64 -I$XMOS_TOOL_PATH/include -O3 fast-simulate.c $XMOS_TOOL_PATH/lib/libxsidevice.so -o fast-simulator

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "xsidevice.h"

#define NREACTIONS  32
int main(int argc, char *argv[]) {
    int status, i, errors;
    int error = 0;
    int next_pulse_at = 6000;
    int clk = 0;
    void *xsim;
    int clockcount = 0;
    int reactions[NREACTIONS];
    for(int i = 0; i < NREACTIONS; i++) {
        reactions[i] = 0;
    }

    status = xsi_create(&xsim, argv[1]);
    assert(status == XSI_STATUS_OK);

    fprintf(stderr, "Simulating\n");
    for(int i = 0; i < 400; i++) {
        // Clock simulator until input needs toggling
        while(clockcount != next_pulse_at) {
            status = xsi_clock(xsim);
            assert(status == XSI_STATUS_OK || status == XSI_STATUS_DONE);
            clockcount++;
        }
        // Toggle input pin
        status = xsi_drive_port_pins(xsim, "tile[0]", "XS1_PORT_1B", 1, 1);
        assert(status == XSI_STATUS_OK || status == XSI_STATUS_DONE );
        int ps = clockcount;
        for(int k = 0; k < 15; k++) {
            status = xsi_clock(xsim);
            assert(status == XSI_STATUS_OK || status == XSI_STATUS_DONE);
            clockcount++;
        }
        status = xsi_drive_port_pins(xsim, "tile[0]", "XS1_PORT_1B", 1, 0);
        assert(status == XSI_STATUS_OK || status == XSI_STATUS_DONE );
        // Now wait for output to be toggled
        unsigned samp;
        do {
            status = xsi_clock(xsim);
            assert(status == XSI_STATUS_OK || status == XSI_STATUS_DONE);
            clockcount++;
            status = xsi_sample_port_pins(xsim, "tile[0]", "XS1_PORT_1A", 1, &samp);
            assert(status == XSI_STATUS_OK || status == XSI_STATUS_DONE);
        } while(samp == 0);
        reactions[clockcount - next_pulse_at]++;
        if (i & 1) {
            next_pulse_at = clockcount + 1000 + i/2;
        } else {
            next_pulse_at = ps + 1000 + i/2;
        }
        // done
    }
    for(int i = 0; i < NREACTIONS; i++) {
        if (reactions[i]) {
            printf("Reaction time %d: %d times\n", i, reactions[i]);
        }
    }
}
